% This script is for cluster-based replication of MC experiments in 
% "Pooled Bewley Estimator of Long Run Relationships in Dynamic Heterogenous Panels"
% by A. Chudik, M.H. Pesaran and R. Smith, May 2021.
% See ReadMe.txt for further details

%% initial settings
R=2000; % number of MC replications
nall=[30,50,100, 200]; % choices for cross section dimension
Tall=[ 31,51,101, 201]; % choices for Te+1 (effective estimation sample size is Te due to one lag)

%% main program

% generate coefs
rng(123456987);
nmax=max(nall); % largest n
alphaall=rand(nmax,1)*0.1+0.2;
beta=1;
sgxall=(rand(nmax,1)*0.4+0.8).^0.5;
sgyall=(rand(nmax,1)*0.4+0.8).^0.5;
muall=randn(nmax,1,2)+ones(nmax,1)*[1,1];  
rhoall=rand(nmax,1)*0.4+0.3;
cchlall=(ones(nmax,1)-rhoall.^2).^0.5;
ni=0;
rr=floor(rand(R,1)*(10^9));

for n=nall
    ni=ni+1;
    nt=0;
    alpha=alphaall(1:n);
    sgx=sgxall(1:n); SGx=diag(sgx);
    sgy=sgyall(1:n); SGy=diag(sgy);
    mu=muall(1:n);
    rho=rhoall(1:n);
    cchl=cchlall(1:n);
for T=Tall
    nt=nt+1;
    [n,T] % displays [n,T] choice (efective sample size is Te=T-1, due to one lag)

    
%initialize matrices where results will be stored    
F1=zeros(R,1); F2=zeros(R,1); F3=zeros(R,1); F4=zeros(R,1); F5=zeros(R,1); F6=zeros(R,1); F7=zeros(R,1); F8=zeros(R,1);
S1=zeros(R,1); S2=zeros(R,1); S3=zeros(R,1); S4=zeros(R,1); S5=zeros(R,1); S6=zeros(R,1); S7=zeros(R,1); S8=zeros(R,1); 
P1=zeros(R,1); P2=zeros(R,1); P3=zeros(R,1); P4=zeros(R,1); P5=zeros(R,1); P6=zeros(R,1); P7=zeros(R,1); P8=zeros(R,1);
% 1 for pooled Bewley
% 2 for PMG
% 3 for jackknife bias-corrected pooled Bewley
% 4 for simulation bias-corrected pooled Bewley
% 5 for combined bias-corrected pooled Bewley
% 6 for jackknife corrected PMG
% 7 for simulation bias-corrected PMG
% 8 for combined bias-corrected PMG

parfor r=1:R 
    rng(rr(r))
    a=[alpha.*mu(:,1)-beta*alpha.*mu(:,2),zeros(n,1)];
    
% generate data
x=zeros(T,n)+ones(T,1)*mu(:,2)';y=zeros(T,n)+ones(T,1)*mu(:,1)';
ey=zeros(T,n);
ex=zeros(T,n);
for i=1:n
    e=randn(T,2);
    C=[1,0;rho(i,1),cchl(i,1)];
    e=e*C';
    ey(:,i)=e(:,1);
    ex(:,i)=e(:,2);
end
uy=ey*SGy;
ux=ex*SGx;
sgp=sgy.^2+beta*beta*(sgx.^2)-2*beta*sgx.*sgy.*rho;
svar=sgp./(1-(1-alpha).^2);
y(1,:)=y(1,:)+randn(1,n).*(svar'.^0.5) ;
for t=2:T
  x(t,:)=x(t-1,:)+ux(t,:);  
  y(t,:)=y(t-1,:)+a(:,1)'-alpha'.*(y(t-1,:)-beta*x(t-1,:))+uy(t,:);  
end

% conduct estimations
[betahat1, betavar1]=ardlpmg_v(y,x); %pooled Bewley
p=1; % lags in level specification (p-1) in first difference specification
[betahat2,stdbeta2,iter, converge]=PMGe(y,x,p); % PMG
betavar2=stdbeta2^2;
kappa=1/3; [betahat3, betavar3]=ardlpmg_vj(y,x, kappa, 1); %pooled Bewley jackknife
[betahat5, betavar5bc,betavar5,kappahat, CVs ]=ardlpmg_bc(y,x); % pooled Bewley simul based bias corrected
kappa=kappahat; [betahat4, betavar4]=ardlpmg_vj(y,x, kappa,1); % pooled Bewley combined (simul jackknife)
[PMGresults ]=pmg_bc(y,x); % PMG bias-corrected

% collect results
    % for mean and bias
    F1(r,1)=betahat1;
    F2(r,1)=betahat2;
    F3(r,1)=betahat3;
    F4(r,1)=betahat4;
    F5(r,1)=betahat5;
    betahat6=PMGresults.jack13.betahat; betavar6=PMGresults.jack13.var;
    betahat7=PMGresults.jackhat.betahat; betavar7=PMGresults.jackhat.var;
    betahat8=PMGresults.simulbased.betahat; betavar8=PMGresults.simulbased.var;
    F6(r,1)=betahat6; 
    F7(r,1)=betahat7;
    F8(r,1)=betahat8;
  
    % for size and power
    if abs(betahat1-1)/(betavar1^0.5) > 1.96
        S1(r,1)=1;
    end
    if abs(betahat2-1)/(betavar2^0.5) > 1.96
        S2(r,1)=1;
    end
    if abs(betahat3-1)/(betavar3^0.5) > CVs.jack13_jf
        S3(r,1)=1;
    end
    if abs(betahat4-1)/(betavar4^0.5) > CVs.jackhat_jf
        S4(r,1)=1;
    end   
    if abs(betahat5-1)/(betavar5^0.5) > CVs.simulbased
        S5(r,1)=1;
    end  
    if abs(betahat6-1)/(betavar6^0.5) > PMGresults.jack13.cv
        S6(r,1)=1;
    end
    if abs(betahat7-1)/(betavar7^0.5) > PMGresults.jackhat.cv
        S7(r,1)=1;
    end
    if abs(betahat8-1)/(betavar8^0.5) > PMGresults.simulbased.cv
        S8(r,1)=1;
    end
    
  power_alt=0.98; % alternative for power
    if abs(betahat1-power_alt)/(betavar1^0.5) > 1.96
        P1(r,1)=1;
    end
    if abs(betahat2-power_alt)/(betavar2^0.5) > 1.96
        P2(r,1)=1;
    end
    if abs(betahat3-power_alt)/(betavar3^0.5) > CVs.jack13_jf
        P3(r,1)=1;
    end
    if abs(betahat4-power_alt)/(betavar4^0.5) > CVs.jackhat_jf
        P4(r,1)=1;
    end
    if abs(betahat5-power_alt)/(betavar5^0.5) > CVs.simulbased
        P5(r,1)=1;
    end
    if abs(betahat6-power_alt)/(betavar6^0.5) > PMGresults.jack13.cv
        P6(r,1)=1;
    end
    if abs(betahat7-power_alt)/(betavar7^0.5) > PMGresults.jackhat.cv
        P7(r,1)=1;
    end
    if abs(betahat8-power_alt)/(betavar8^0.5) > PMGresults.simulbased.cv
        P8(r,1)=1;
    end
    
end %r

bias1(ni,nt)=mean(F1)-beta; 
bias2(ni,nt)=mean(F2)-beta;
bias3(ni,nt)=mean(F3)-beta;
bias4(ni,nt)=mean(F4)-beta;
bias5(ni,nt)=mean(F5)-beta;
bias6(ni,nt)=mean(F6)-beta;
bias7(ni,nt)=mean(F7)-beta;
bias8(ni,nt)=mean(F8)-beta;

rmse1(ni,nt)=((F1-beta)'*(F1-beta)/R)^0.5;
rmse2(ni,nt)=((F2-beta)'*(F2-beta)/R)^0.5;
rmse3(ni,nt)=((F3-beta)'*(F3-beta)/R)^0.5;
rmse4(ni,nt)=((F4-beta)'*(F4-beta)/R)^0.5;
rmse5(ni,nt)=((F5-beta)'*(F5-beta)/R)^0.5;
rmse6(ni,nt)=((F6-beta)'*(F6-beta)/R)^0.5;
rmse7(ni,nt)=((F7-beta)'*(F7-beta)/R)^0.5;
rmse8(ni,nt)=((F8-beta)'*(F8-beta)/R)^0.5;

sizemc1(ni,nt)=mean(S1);
sizemc2(ni,nt)=mean(S2);
sizemc3(ni,nt)=mean(S3);
sizemc4(ni,nt)=mean(S4);
sizemc5(ni,nt)=mean(S5);
sizemc6(ni,nt)=mean(S6);
sizemc7(ni,nt)=mean(S7);
sizemc8(ni,nt)=mean(S8);

power1(ni,nt)=mean(P1);
power2(ni,nt)=mean(P2);
power3(ni,nt)=mean(P3);
power4(ni,nt)=mean(P4);
power5(ni,nt)=mean(P5);
power6(ni,nt)=mean(P6);
power7(ni,nt)=mean(P7);
power8(ni,nt)=mean(P8);

end %n
end%T

save('MC_results1.mat'); % saves workspace (all results) to mat file
disp('MC job finished')